;;; ENV-EVAL-CBN.SCM
;;;
;;; Evaluation of INTEX+BIND programs using the 
;;; call-by-name environment model. In this
;;; model, environment maps names to *thunks*
;;; (zero-parameter functions) that return 
;;; the value when called. 

;; Invoke a program on any number of integer arguments
(define env-run-cbn
  (lambda (pgm ints)
    (env-eval-cbn (program-body pgm)
		  (env-make (program-formals pgm) 
			    ;; Make thunk for each param
			    (map (lambda (n) (lambda () n))
				 ints)))))

;; Evaluate EXP relative to the environment ENV. 
;; The environment ENV is a collection of delayed substitutions.
(define env-eval-cbn
  (lambda (exp env)
    (cond ((literal? exp) 
           (literal-value exp))
          ((varref? exp) 
           (let ((probe (env-lookup (varref-name exp) env)))
             (if (unbound? probe)
               (throw 'unbound-variable (varref-name exp))
	       ;; Force the evaluation of thunk.
               (probe))))
          ((binapp? exp)
           ((binop-to-function (binapp-rator exp))
            (env-eval-cbn (binapp-rand1 exp) env)
            (env-eval-cbn (binapp-rand2 exp) env)))
          ((bind? exp)
           (env-eval-cbn 
	    (bind-body exp)
	    (env-bind (bind-name exp)
		      ;; Bind name to thunk of evaluation
		      (lambda () (env-eval-cbn (bind-defn exp) env))
		      env)))
	  (else (error "ENV-EVAL: Unknown expression type -- " exp))
          )))

;; Use BINOP-TO-FUNCTION from SUBST-EVAL.SCM





