;;; ENV-EVAL-CBV.SCM
;;;
;;; Call-by-value evaluation of INTEX+BIND programs 
;;; using the environment model

;; Invoke a program on any number of integer arguments
(define env-run-cbv
  (lambda (pgm ints)
    (env-eval-cbv (program-body pgm)
              (env-make (program-formals pgm) ints))))

;; Evaluate EXP relative to the environment ENV. 
;; The environment ENV is a collection of delayed substitutions.
(define env-eval-cbv
  (lambda (exp env)
    (cond ((literal? exp) 
           (literal-value exp))
          ((varref? exp) 
           (let ((probe (env-lookup (varref-name exp) env)))
             (if (unbound? probe)
		 (throw 'unbound-variable (varref-name exp))
		 probe)))
          ((binapp? exp)
           ((binop-to-function (binapp-rator exp))
            (env-eval-cbv (binapp-rand1 exp) env)
            (env-eval-cbv (binapp-rand2 exp) env)))
          ((bind? exp)
           (env-eval-cbv (bind-body exp)
                     (env-bind (bind-name exp)
                               (env-eval-cbv (bind-defn exp) env)
                               env)))
           (else (error "ENV-EVAL: Unknown expression type -- " exp))
          )))

;; Use BINOP-TO-FUNCTION from SUBST-EVAL.SCM
