;;; FREE-BINDING-BOUND-VARS.SCM for INTEX+BIND programs

;;; Functions for calculating the variables which have
;;; free occurrences in INTEX+BIND programs and expressions.

(define free-vars-program
  (lambda (pgm) 
    (set-difference (free-vars (program-body pgm))
                    (list->set (program-formals pgm)))))

;; Return the free vars of a INTEX+BIND expression

(define free-vars
  (lambda (exp) 
    (cond ((literal? exp) 
           (set-empty))
          ((varref? exp) 
           (set-singleton (varref-name exp)))
          ((binapp? exp)
           (set-union (free-vars (binapp-rand1 exp))
                      (free-vars (binapp-rand2 exp))))
	  ((bind? exp)
	   (set-union (free-vars (bind-defn exp))
		      (set-difference (free-vars (bind-body exp))
				      (set-singleton (bind-name exp)))))
          (else (error "FREE-VARS: unrecognized expression -- " exp))
          )))

;;; BINDING-VARS
;;;
;;; Functions for calculating the variables which have
;;; binding occurrences in INTEX+BIND programs and expressions.

(define binding-vars-program
  (lambda (pgm) 
    (set-union (binding-vars (program-body pgm))
	       (list->set (program-formals pgm)))))

;; Return vars with binding occurrences in a INTEX+BIND expression

(define binding-vars
  (lambda (exp) 
    (cond ((literal? exp) 
           (set-empty))
          ((varref? exp) 
           (set-empty))
          ((binapp? exp)
           (set-union (binding-vars (binapp-rand1 exp))
                      (binding-vars (binapp-rand2 exp))))
	  ((bind? exp)
	   (set-union (binding-vars (bind-body exp))
		      (set-singleton (bind-name exp))))
          (else (error "BINDING-VARS: unrecognized expression -- " exp)))))

;;; BOUND-VARS
;;;
;;; Functions for calculating the variables which have
;;; bound occurrences in INTEX+BIND programs and expressions.

(define bound-vars-program
  (lambda (pgm) 
    (bound-vars (program-body pgm))))


;; Return vars with bound occurrences in a INTEX+BIND expression

(define bound-vars
  (lambda (exp) 
    (cond ((literal? exp) 
           (set-empty))
          ((varref? exp) 
           (set-empty))
          ((binapp? exp)
           (set-union (bound-vars (binapp-rand1 exp))
                      (bound-vars (binapp-rand2 exp))))
	  ((bind? exp)
	   (if (set-member? (bind-name exp) (free-vars (bind-body exp)))
	       (set-union (bound-vars (bind-body exp))
			  (set-singleton (bind-name exp)))
	       (bound-vars (bind-body exp))))
          (else (error "BOUND-VARS: unrecognized expression -- " exp)))))
