;;; INTEX+BIND-EXAMPLES.SCM

;;; Examples of INTEX programs written in s-expression syntax

(define inc
  '(program (x) (+ x 1)))

;; Centigrade to Fahrenheit
(define c2f
  '(program (cent)
     (+ 32
        (div (* 9 cent)
             5))))

;; Fahrenheit to Centigrade
(define f2c
  '(program (fahr)
     (div (* 5 (- fahr 32))
          9)))

;;; Examples of INTEX+BIND programs written in s-expression syntax

(define c2f1
  '(program (cent)
     (bind freezing 32
       (+ freezing
          (div (* 9 cent)
               5)))))

;; Calculate  (a^2 + b^2) / (a^2 - b^2)
(define calc
  '(program (a b)
     (bind a_sq (* a a)
       (bind b_sq (* b b)
         (bind numer (+ a_sq b_sq)
           (bind denom (- a_sq b_sq)
	     (div numer denom)))))))

;; Calculate  ((a + 1)^2 + (a - 1)^2)/((a + 1)^2 - (a - 1)^2)
;; Note: this calculates (2a^2 + 2)/4a
(define calc2
  '(program (a)
     (bind a_inc (+ a 1)
       (bind a_dec (- a 1)
         (bind a_inc_squared (* a_inc a_inc)
           (bind a_dec_squared (* a_dec a_dec)
             (div (+ a_inc_squared a_dec_squared)
                  (- a_inc_squared a_dec_squared))))))))

;; Two-argument program
(define avg
  '(program (a b) 
     (bind c (+ a b)
       (div c 2))))

;;; Examples of BINDEX = INTEX+BIND+BINDSEQ+BINDPAR
;;; programs written in s-expression syntax

;; Bindseq tests
; (define test-bindseq
;  '(program (x) ;  Suppose x = 3
;     (bind a (+ x 1) ; a = 4
;       (bindseq ((a (* x 10)) ; a = 30
;                 (b (+ x a))) ; b = 33
;         (+ a b))))) ; 63

; (define test-bindseq2
;  '(program (x) ;  Suppose x = 3
;     (bind a (+ x 1) ; a = 4
;       (bindseq ((a (* x a))  ; a = 12
;                 (b (+ x a))) ; b = 15
;         (bindseq ((a (- a b)) ; a = -3
;                   (b (* a a))) ; b = 9
;           (+ a b)))))) ; 6

;; Bindpar tests
; (define test-bindpar
;  '(program (x) ;  Suppose x = 3
;     (bind a (+ x 1) ; a = 4
;       (bindpar ((a (* x 10))  ; a = 30
;                 (b (+ x a))) ; b = 7
;         (+ a b))))) ; 37

; (define test-bindpar2
;  '(program (x) ;  Suppose x = 3
;     (bind a (+ x 1) ; a = 4
;       (bindpar ((a (* x a))  ; a = 12
;                 (b (+ x a))) ; b = 7
;         (bindpar ((a (- a b)) ; a = 5
;                   (b (* a a))) ; b = 144
;           (+ a b)))))) ; 149  

;; AST test
;; E.g. (env-run-cbn test-ast '(5 4 2))
(define test-ast
  '(program (a b c)
     (* (bind d (* a c) 
           (bind e (- d b) 
              (div (* b d) 
                   (+ e a))))
         (bind e (bind b (* 12 a) 
                     (- b c)) 
            (div e b))))) 

(define test-unbound
  '(program (a b)
      (* (+ a b) c)))

(define test-div-by-zero
  '(program (a b c)
      (div (+ a b) c)))

(define test-mod-by-zero
  '(program (a b c)
      (mod (+ a b) c)))

(define test-unbound2
  ;; This can go bad if CBN subst is  not working correctly
  ;; Note that the "going bad" can only be illustrated
  ;; with a free program variable.
  '(program ()
     (bind a (+ b 1)
       (bind b (* 2 3)
         (* a a)))))





