;;; SUBST-EVAL-CBV.SCM
;;; 
;;; Call-by-value evaluation of INTEX+BIND programs 
;;; using the substitution model

;; Invoke a program on any number of integer values
(define subst-run-cbv
  (lambda (pgm ints)
    (literal-value
     (subst-eval-cbv (subst* (map make-literal ints)
			     (program-formals pgm)
			     (program-body pgm))))))

;; SUBST-EVAL-CBV takes an expression and returns another expression 
;; (e.g., an integer literal node, *not* an integer) that is the
;; result of evaluting the expression. 
(define subst-eval-cbv
  (lambda (exp)
    (cond ((literal? exp) exp)
          ((varref? exp) 
           (throw 'unbound-variable (varref-name exp)))
          ((binapp? exp)
           ((binop-to-node-function (binapp-rator exp))
            (subst-eval-cbv (binapp-rand1 exp))
            (subst-eval-cbv (binapp-rand2 exp))))
          ((bind? exp)
           (subst-eval-cbv
            (subst1 (subst-eval-cbv (bind-defn exp))
		    (bind-name exp)
		    (bind-body exp))))
          (else (error "SUBST-EVAL-CBV: Unknown expression type -- " exp))
          )))

;; Returns a Scheme function that takes two integer
;; literal nodes and returns a integer literal node
;; whose value is the results of applying the 
;; specified operator to the values of the two arguments.
(define binop-to-node-function
  (lambda (binop)
    (lambda (lit1 lit2)
      (make-literal 
        ((binop-to-function binop)
         (literal-value lit1)
         (literal-value lit2))))))

;; Returns a Scheme int-to-int function indicated by the symbol
;; Given the symbol name for a binary operator, 
;; return the corresponding Scheme procedure.
(define binop-to-function
  (lambda (binop)
    (cond ((eq? binop '+) +)
          ((eq? binop '-) -)
          ((eq? binop '*) *)
          ((eq? binop 'div) 
           (lambda (a b)
             (if (= b 0)
                 (throw 'division-by-zero (list 'div a b))
                 (quotient a b))))
          ((eq? binop 'mod) 
           (lambda (a b)
             (if (= b 0)
                 (throw 'division-by-zero (list 'mod a b))
                 (remainder a b))))
          (else (error "BINOP-TO-FUNCTION: unrecognized binop " binop))
          )))

                                  

              
              
